precision highp float;
uniform sampler2D inputImageTexture;
varying highp vec2 textureCoordinate;
uniform vec3 iResolution;
uniform vec4 iMouse;
uniform int blurFactor;
uniform float blurFactor1;
uniform float blurFactor2;
uniform float mouseSize;

float smoothObjectPadding = 0.33;


const float TEXTURE_SIZE = 512.0;
const float PIXEL_SIZE = (1.0 / TEXTURE_SIZE);

float gaussianExpression(float sigma, float xy)
{
    return exp(-(xy * xy) / (2.0 * sigma * sigma));
}

void mainImage( out vec4 fragColor, in vec2 fragCoord )
{
    vec2 uv = fragCoord.xy/iResolution.xy;
    vec3 color = texture2D(inputImageTexture, uv).rgb;
    float sigma = 5.0;
    float sum = 0.0;
    vec3 ret = vec3(0);
    int center = blurFactor/2;
    for (int y = 0; y < blurFactor; ++y) {
        float float_y = gaussianExpression(sigma, float(y) - float(center));
        float offset_y = float(y - center) * PIXEL_SIZE;
        for (int x = 0; x < blurFactor; ++x) {
            float float_x = gaussianExpression(sigma, float(x) - float(center));
            float offset_x = float(x - center) * PIXEL_SIZE;
            sum += float_x * float_y;
            ret += texture2D(inputImageTexture, uv + vec2(offset_x*blurFactor1, offset_y*blurFactor1),blurFactor2).rgb * float_x * float_y;
        }
    }
    vec3 blurColor = ret / sum;

    float aspectRatio = iResolution.x / iResolution.y;
    vec2 objectCenter = iMouse.xy/iResolution.xy;

    vec2 v = uv - objectCenter;
    v.x = 0.0;
    float size = mouseSize / 2.0;
    float smoothSize = size * smoothObjectPadding;
    float squareMix = clamp(length(smoothstep(size - smoothSize, size, abs(v))), 0.0, 1.0);
    blurColor.rgb = mix(color.rgb,blurColor.rgb, squareMix);

    if(blurFactor==0){
        fragColor = vec4(color, 1.0);
    }else{
        fragColor = vec4(blurColor, 1.0);
    }
}


void main(){
    mainImage(gl_FragColor, textureCoordinate * iResolution.xy );
}