precision highp float;
uniform sampler2D inputImageTexture;
varying highp vec2 textureCoordinate;
uniform vec3 iResolution;
uniform vec4 iMouse;
uniform float grides;

uniform float mouseSize;
float smoothObjectPadding = 0.33;

void mainImage( out vec4 fragColor, in vec2 fragCoord )
{
    vec2 uv = fragCoord/iResolution.xy;
    vec4 color = texture2D(inputImageTexture, uv);

    float aspectRatio = iResolution.x / iResolution.y;
    vec2 objectCenter = iMouse.xy/iResolution.xy;

    vec2 v = uv - objectCenter;
    v.x = v.x * aspectRatio;

    float size = mouseSize / 2.0;
    float smoothSize = size * smoothObjectPadding;
    float circleMix = smoothstep( size - smoothSize, size,length(v));

    vec2 res = vec2(aspectRatio / grides, 1.0 / grides);
    vec2 pixUv = floor(uv * res ) / res;
    vec4 pixColor = texture2D(inputImageTexture, pixUv);
    color.rgb = mix(color.rgb,pixColor.rgb, circleMix);

    fragColor = color;
}

void main(){
    mainImage(gl_FragColor, textureCoordinate * iResolution.xy );
}